static char *sccs_id= (sccs_id, "%W% %G%");

/*
 * This file implements a GeoShape which draws raster maps which
 * are created with the raster/makemaps script.
 */

#include "Class.h"
#include "IconShape.h"
#include "OverView.h"


class RastShape: public QueryShape {
  int	size_x, size_y;
  byte	*ba;
public:
  RastShape(MapView *, int n, char **str);
  ~RastShape() { delete ba; }

  // void CompBBox();
  void Draw();
};


extern "C" RastShape *make_rast_shape(MapView *dv, int n, char **str)
{
  RastShape *s= new RastShape(dv, n, str);
  return s;
}


RastShape::RastShape(MapView *d, int n, char **str): (d)
{
  if (n != 4) {
    fprintf(stderr, "RastShape: bad number of arguments: %d", n);
    return;
  }

  float p1_x, p1_y, p2_x, p2_y;
  if (sscanf(str[2], "(%f,%f,%f,%f)", &p1_x, &p1_y, &p2_x, &p2_y) != 4) {
    fprintf(stderr, "RastShape: bad bbox !\n");
    return;
  }

  size_x= atoi(str[0]);
  size_y= atoi(str[1]);

  int size= size_x * size_y;
  ba= new byte[size];
  bcopy(str[3], ba, size);

  labelpos= d->GeoToPoint(p2_x, p2_y);
  SetBBox(NormRect(d->GeoToPoint(p1_x, p1_y), d->GeoToPoint(p2_x, p2_y)));
}


void RastShape::Draw()
{
  if (!size_x || bbox.extent.y == 0 || bbox.extent.x == 0)
    return;

GrGiveHint(eHintBatch, sizeof(Rectangle), &bbox);

  float	pw= bbox.extent.x/float(size_x);
  float	ph= bbox.extent.y/float(size_y);

  if (pw < 1 || ph < 1)
    return;

  int pwround= pw+1.0;
  int phround= ph+1.0;
  float xstart= bbox.origin.x+0.5;
  float ystart= bbox.origin.y+0.5;

  float x, y;
  int	xi, yi;
  byte	*p= ba;

  for (y= ystart, yi= 0; yi < size_y; y+= ph, yi++) {
    for (x= xstart, xi= 0; xi < size_x; x+= pw, xi++) {
      Rectangle r(x,y,pwround,phround);

      GrPaintRect(r, GetInk(*p - '0'));
      p++;
    }
  }

GrGiveHint(eHintTextBatch);
}
